package pps.core.base.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.core.base.entity.BasePowerLineEnt;
import pps.core.base.entity.BasePowerLineView;
import pps.core.base.mapper.BasePowerLineMapper;
import pps.core.base.mapper.BasePowerLineViewMapper;
import pps.core.base.service.data.base_power_line.*;
import pps.core.common.session.PpsUserSession;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.data.CustomQueryInput;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;
import xstartup.helper.XTransactionHelper;

import java.util.List;

/**
 * 输电线路配置模块
 *
 * @author ZWT
 * @date 2023/08/25
 */
@XService
@XText("输电线路配置模块")
public class BasePowerLineService {

    /**
     * 输电线路配置模块--新增
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XApiAnonymous
    @XApiPost
    @XText("输电线路配置模块--新增")
    public XServiceResult createBasePowerLine(XContext context, CreateBasePowerLineInput input) {
        //        PpsUserSession session = context.getSession(PpsUserSession.class);
        PpsUserSession session = new PpsUserSession();
        session.setId("123");
        session.setUserName("ceshi");
        return XTransactionHelper.begin(context, () -> {
            BasePowerLineMapper mapper = context.getBean(BasePowerLineMapper.class);
            BasePowerLineEnt entity = new BasePowerLineEnt();
            XCopyUtils.copyObject(input, entity);
            mapper.insert(entity);
            return XServiceResult.OK;
        });
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult updateBasePowerLine(XContext context, UpdateBasePowerLineInput input) {
        BasePowerLineMapper mapper = context.getBean(BasePowerLineMapper.class);
        QueryWrapper<BasePowerLineEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasePowerLineEnt::getId, input.getId());
        BasePowerLineEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult deleteBasePowerLine(XContext context, DeleteBasePowerLineInput input) {
        BasePowerLineMapper mapper = context.getBean(BasePowerLineMapper.class);
        QueryWrapper<BasePowerLineEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasePowerLineEnt::getId, input.getId());
        BasePowerLineEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetBasePowerLineOutput> getBasePowerLine(XContext context, GetBasePowerLineInput input) {
        BasePowerLineMapper mapper = context.getBean(BasePowerLineMapper.class);
        QueryWrapper<BasePowerLineEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasePowerLineEnt::getId, input.getId());
        BasePowerLineEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBasePowerLineOutput output = new GetBasePowerLineOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryBasePowerLineOutput> queryBasePowerLine(XContext context, QueryBasePowerLineInput input) {
        BasePowerLineMapper mapper = context.getBean(BasePowerLineMapper.class);
        QueryWrapper<BasePowerLineEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().like(BasePowerLineEnt::getId, input.getId())
                .eq(BasePowerLineEnt::getIsDeleted, input.getIsDeleted())
                .like(BasePowerLineEnt::getCreateById, input.getCreateById())
                .like(BasePowerLineEnt::getCreateByName, input.getCreateByName())
                .eq(BasePowerLineEnt::getCreateTime, input.getCreateTime())
                .like(BasePowerLineEnt::getModifyById, input.getModifyById())
                .like(BasePowerLineEnt::getModifyByName, input.getModifyByName())
                .eq(BasePowerLineEnt::getModifyTime, input.getModifyTime())
                .like(BasePowerLineEnt::getOuId, input.getOuId())
                .like(BasePowerLineEnt::getUpperLineId, input.getUpperLineId())
                .like(BasePowerLineEnt::getLineName, input.getLineName())
                .like(BasePowerLineEnt::getGridTypeKey, input.getGridTypeKey())
                .like(BasePowerLineEnt::getLineTypeKey, input.getLineTypeKey())
                .like(BasePowerLineEnt::getStrategyId, input.getStrategyId())
                .eq(BasePowerLineEnt::getIsShareDispatch, input.getIsShareDispatch())
                .eq(BasePowerLineEnt::getIsReverseDispatch, input.getIsReverseDispatch());
        return XMapperHelper.query(mapper, input, queryWrapper, QueryBasePowerLineOutput.class);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryBasePowerLineOutput> dynamicQueryBasePowerLine(XContext context, CustomQueryInput input) {
        BasePowerLineMapper mapper = context.getBean(BasePowerLineMapper.class);
        return XMapperHelper.query(mapper, input, BasePowerLineEnt.class, QueryBasePowerLineOutput.class);
    }

    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetBasePowerLineViewOutput> getBasePowerLineView(XContext context, GetBasePowerLineViewInput input) {
        BasePowerLineViewMapper mapper = context.getBean(BasePowerLineViewMapper.class);
        BasePowerLineView record = new BasePowerLineView();
        XCopyUtils.copyObject(input, record);
        BasePowerLineView view = mapper.selectOne(record);
        if (view == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBasePowerLineViewOutput output = new GetBasePowerLineViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryBasePowerLineViewOutput> queryBasePowerLineView(XContext context, QueryBasePowerLineViewInput input) {
        BasePowerLineViewMapper mapper = context.getBean(BasePowerLineViewMapper.class);
        BasePowerLineView record = new BasePowerLineView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<BasePowerLineView> list = mapper.selectList(record);
        PageInfo<BasePowerLineView> pageInfo = new PageInfo<>(list);
        List<QueryBasePowerLineViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBasePowerLineViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

}
